TwoPTwoPGraphError

Enum TwoPTwoPGraphError 

pub enum TwoPTwoPGraphError<Id>
where Id: Debug,
{ VertexAlreadyExists(Id), VertexDoesNotExists(Id), EdgeAlreadyExists(Id), EdgeDoesNotExists(Id), VertexHasEdge(Id, Id), AddVertexNotDelivered(Id), AddEdgeNotDelivered(Id), }
Errors that can occur when performing operations on a TwoPTwoPGraph.

Variants



VertexAlreadyExists(Id)

A vertex with the given ID has already been added.



VertexDoesNotExists(Id)

The referenced vertex does not exist in V_A \ V_R.



EdgeAlreadyExists(Id)

An edge with the given ID has already been added.



EdgeDoesNotExists(Id)

The referenced edge does not exist in E_A \ E_R.



VertexHasEdge(Id, Id)

Cannot remove the vertex because it still has an active (non-removed) edge.



AddVertexNotDelivered(Id)

Downstream precondition failure: the corresponding addVertex has not been delivered yet.



AddEdgeNotDelivered(Id)

Downstream precondition failure: the corresponding addEdge has not been delivered yet.