pub type Graph = TwoPTwoPGraph<AddVertex, RemoveVertex, AddEdge, RemoveEdge, Uuid>;Expand description
A graph whose vertices and edges may carry binary data.
Aliased Type
pub struct Graph { /* private fields */ }pub type Graph = TwoPTwoPGraph<AddVertex, RemoveVertex, AddEdge, RemoveEdge, Uuid>;A graph whose vertices and edges may carry binary data.
pub struct Graph { /* private fields */ }